#include <QtGui>

#include "FormMain.h"

//  
FormMain::FormMain()
{
	//       
	setModified(false);						// ,     
	ProgramTitle = tr("Birthdays");			//  
	ProgramVersion = "0.5";					//  
	ProgramDate = "23 Sep 2011";			//    
	ProgramSite = "birthdays.shagov.info";	//  
	HomeDir = QDir::homePath() + "/";	//     (  )

	//    UI         
	QWidget *w = new QWidget;
	setupUi(w);
	setCentralWidget(w);

	//    
	ButtonAdd->setIcon(		QIcon(":/images/add.png") );
	ButtonDelete->setIcon(	QIcon(":/images/delete.png") );
	ButtonChange->setIcon(	QIcon(":/images/change.png") );
	ButtonInfo->setIcon(	QIcon(":/images/info.png") );
	ButtonAbout->setIcon(	QIcon(":/images/about.png") );
	ButtonSettings->setIcon(QIcon(":/images/settings.png") );
	ButtonRemind->setIcon(	QIcon(":/images/remind.png"));
	ButtonOK->setIcon(		QIcon(":/images/ok.png") );

	//    
	connect( ButtonAdd,			SIGNAL(clicked()), this, SLOT(addPerson()) );
	connect( ButtonDelete,		SIGNAL(clicked()), this, SLOT(deletePerson()) );
	connect( ButtonChange,		SIGNAL(clicked()), this, SLOT(changePerson()) );
	connect( ButtonInfo,		SIGNAL(clicked()), this, SLOT(infoPerson()) );
	connect( ButtonAbout,		SIGNAL(clicked()), this, SLOT(about()) );
	connect( ButtonSettings,	SIGNAL(clicked()), this, SLOT(settings()) );
	connect( ButtonRemind,		SIGNAL(clicked()), this, SLOT(remindFromForm()) );
	connect( ButtonOK,			SIGNAL(clicked()), this, SLOT(ok()) );

	connect( ListMonth,	SIGNAL(currentRowChanged(int)),		this, SLOT( showMonth(int) ) );
	connect( TabWidget,	SIGNAL(currentChanged(int)),		this, SLOT( tabChanged(int) ) );
	connect( Table,		SIGNAL(cellDoubleClicked(int, int)),this, SLOT( doubleClicked(int, int) ) );

	//      
	Table->setColumnCount(3);
	Table->setRowCount(0);
	Table->setHorizontalHeaderLabels( QStringList() << tr("Name") << "ComputerDate" << tr("Date") ); //   
	Table->verticalHeader()->hide();								//    
	Table->setEditTriggers( QAbstractItemView::NoEditTriggers );	//    
	Table->setSelectionBehavior( QAbstractItemView::SelectRows );	//    ,   
	Table->setSelectionMode( QAbstractItemView::SingleSelection );	//     
	Table->horizontalHeader()->setStretchLastSection(true);			//     

	//     (  )
	setWindowIcon( QIcon(":/images/icon.png") );
	setWindowTitle( ProgramTitle );

	readSettings();	//     (   Linux),      ,   
	loadFile();		//   
	sortAndSet(ToAll);//   (  ,       )
	//         Date
	createMenu();	//    
}


//    
int FormMain::remind(bool simple)
{
	int r = Table->rowCount();
	//   
	if ( r < 1 )
	{
		if ( simple ) infoRow(); //       ,     
		return 0; //  ,     -r    
	}

	bool IsSorted = false;
	if ( SortBy == 0 )
	{
		SortBy = 1;
		sortAndSet(ToNowhere);
		IsSorted = true;
	}

	bool one = false; //  ,        
	QString list = "";
	for ( int i = 0; i < r; i++ )
	{
		QString pn = Table->item(i,0)->data(0).toString();	//  
		QString md = Table->item(i,1)->data(0).toString();	//  
		QString pd = Table->item(i,2)->data(0).toString();	//     
		int p = md.indexOf(".");		//   .
		int m = md.left(p).toInt();		//  ,    - 
		int d = md.right(p).toInt();	//  ,    - 

		int days = daysToBirthday(m,d); // ,       

		if ( days <= RemindBy ) //       ,   
		{
			QString cday = "";
			if ( days == 0 )
				cday = tr("( today! )");
			else
				cday = "( " + QString::number(days) + " " + tr("d.") + " )";
			list += pd + "\t" + cday + " " + pn + "\n";
			one = true;
		}
	}

	if ( IsSorted ) //    ,     
	{
		SortBy = 0;
		sortAndSet(ToNowhere);
	}

	//   ,   
	if ( !one )
	{
		//       ,    
		if ( simple )
		{
			QMessageBox::information(
							this,
							ProgramTitle,
							tr("No birthdays next %1 d.").arg(QString::number(RemindBy)),
							QMessageBox::Ok,
							QMessageBox::Ok);
		}
		return 0;
	}

	QDate date = QDate::currentDate();
	QString message = tr("Today is %1.%2The nearest birthdays:").arg( date.toString("MMMM, dd") , "\n" ) + "\n\n" + list;

	FormRemind dialog( this, message, simple );
	if ( !simple )
	{
		dialog.setWindowOpacity(0); //    ,     ,   show()   exec()
		dialog.show();
		int x, y;
		QDesktopWidget dw; //           
		QRect rect = dw.availableGeometry();
		x = rect.width()  - dialog.width()  - 13;
		y = rect.height() - dialog.height() - 35;
		dialog.move( x, y );
		dialog.setWindowOpacity(0.975); //    ,      
	}
	//    ,   ,
	//    Open (accept),      
	if ( dialog.exec() )	return 1; //  
	else					return 0; //     
}



//   
void FormMain::closeEvent(QCloseEvent *event)
{
	//      
	if ( getModified() )
	{
		//   -      
		int r = QMessageBox::warning( this, ProgramTitle,
						tr("The table has been modified.%1"
						"Do you want to save changes?").arg("\n"),
						QMessageBox::Yes | QMessageBox::Default,
						QMessageBox::No,
						QMessageBox::Cancel | QMessageBox::Escape );
		switch (r)
		{
			case QMessageBox::Yes	: writeSettings(); saveFile(); event->accept(); break;	//   
			case QMessageBox::No	: event->accept(); break;								//    
			default					: event->ignore(); break;								//     
		}
	}
	else //    
	{
		event->accept(); //    
	}
}



//    
void FormMain::addPerson()
{
	FormAdd dialog( this );
	if ( dialog.exec() ) //       OK
	{
		//   
		QString PersonName = dialog.EditName->text();

		//      
		QString ComputerDate = "";
		int m = dialog.ComboMonth->currentIndex()+1;
		if ( m < 10 ) ComputerDate = "0"; //  0      10 (., 09)
		ComputerDate += QString::number(m) + "." + dialog.ComboDay->currentText(); //    

		// ,       (    ,  ,  -1)
		if ( duplicateName( PersonName, -1 ) )
		{
			addPerson();
			return;
		}

		int last = Table->rowCount();					//     
		Table->setRowCount( Table->rowCount() + 1 );	//   
		setPerson( last, PersonName, ComputerDate );	//      

		Table->setCurrentCell( last, 0 ); //   ,    
		sortAndSet(ToCurrent); //   ( )

		setModified( true ); // ,     ( )      
	}
}



//    
void FormMain::deletePerson()
{
	int cr = Table->currentRow(); //   
	if ( cr < 0 ) //      ,    
	{
		infoRow();
		return;
	}

	QString PersonName = Table->item(cr,0)->data(0).toString();	//       ( )

	//  ,    
	int res = QMessageBox::question( this, tr("Delete"),
	tr("Do you really want to delete %1?").arg( "<strong>" + PersonName + "</strong>" ),
	QMessageBox::Yes | QMessageBox::No, QMessageBox::No );

	//   ,  
	if ( res == QMessageBox::Yes )
	{
		Table->removeRow(cr);
		setModified(true);
	}
}



//    
void FormMain::changePerson()
{
	int cr = Table->currentRow(); //   
	if ( cr < 0 ) //      ,    
	{
		infoRow();
		return;
	}

	FormAdd dialog(this);
	dialog.setWindowIcon( QIcon(":/images/change.png") );

	QString PersonName = Table->item(cr,0)->data(0).toString();	//      ( )
	dialog.EditName->setText( PersonName );						//         
	dialog.setWindowTitle( tr("Change data for %1").arg( PersonName ) ); //   

	//          
	QString ComputerDate = Table->item(cr,1)->data(0).toString();
	int p = ComputerDate.indexOf(".");
	int m = ComputerDate.left(p).toInt() - 1;
	int d = ComputerDate.right(p).toInt() - 1;
	dialog.ComboMonth->setCurrentIndex(m); //     
	dialog.ComboDay  ->setCurrentIndex(d);

	//   ,          
	QString OldPersonName   = PersonName;
	QString OldComputerDate = ComputerDate;

	if ( dialog.exec() )
	{
		//        
		PersonName = dialog.EditName->text();

		//      
		ComputerDate = "";
		int m = dialog.ComboMonth->currentIndex()+1;
		if ( m < 10 ) ComputerDate = "0";
		ComputerDate += QString::number(m) + "." + dialog.ComboDay->currentText();

		if ( duplicateName(PersonName, cr) ) // ,     (     )
		{
			changePerson();
			return;
		}

		setPerson( cr, PersonName, ComputerDate ); //      
		sortAndSet(ToCurrent); //   ( )
		setModified(true);
	}
}



//      
void FormMain::infoPerson()
{
	int cr = Table->currentRow(); //     
	if ( cr < 0 ) //      ,    
	{
		infoRow();
		return;
	}

	//         ,    
	//       
	QString PersonName = Table->item(cr,0)->data(0).toString();	//  
	QString md = Table->item(cr,1)->data(0).toString();			//   (  )

	int p = md.indexOf(".");
	int m = md.left(p).toInt(); //       
	int d = md.right(p).toInt();
	
	FormInfo dialog( this, PersonName, zodiac(m,d), daysToBirthday(m,d) );
	//    ,   ,
	//    Change (accept),    ""
	if ( dialog.exec() ) changePerson();
}



//  
void FormMain::about()
{
	QMessageBox::about(
		this,
		tr("About %1 program").arg("\"" + ProgramTitle + "\""),
		"<h3>" + ProgramTitle + " v" + ProgramVersion + "</h3>" +
		tr(	"With the help of this program you can"
			"%1create and edit the birthdays database."
			"%1It can remind you about the nearest"
			"%1birthdays and show some information"
			"%1about any person in the database.%1%1").arg( "<br />" ) +
			"&copy; <a href='http://" + ProgramSite + "/'>shagov</a> " +
			ProgramDate + "<br /><br />" +
			"<a href='http://" + ProgramSite + "/check/" + ProgramVersion + "'>" + tr("Check for updates") + "</a>");
}



//  
void FormMain::settings()
{
	FormSettings dialog(this); //   
	dialog.EditFileName->setText(FileName); //   -    

	#ifndef Q_OS_WIN //   ,  Windows
		dialog.CheckRemind->setChecked(true);
		dialog.CheckRemind->setEnabled(false);
		dialog.LabelRemind->setEnabled(false);
		dialog.LabelAsterisk->setText(
		"<span style='color: #666; font-size: 7pt;'>" +
		tr("* If you want the program to remind%1 about the birthdays you must start%1 it with \"-r\" parameter").arg("<br />") +
		"</span>" );
	#endif
	#ifdef Q_OS_WIN //  Windows
		dialog.CheckRemind->setChecked( Remind );
		dialog.LabelAsterisk->setText(
		"<span style='color: #666; font-size: 7pt;'>" +
		tr("* The programm will remind%1 you every time it's started").arg("<br />") +
		"</span>" );
	#endif

	QStringList styles = QStyleFactory::keys(); //   
	dialog.ComboBoxStyle->clear();
	dialog.ComboBoxStyle->addItems(styles);
	dialog.ComboBoxStyle->setCurrentIndex( styles.indexOf(ViewStyle) );

	if ( SortBy == 1 )	dialog.RadioDate->setChecked(true); //  
	else				dialog.RadioName->setChecked(true);

	if ( LangRu )	dialog.RadioRu->setChecked(true); //   
	else			dialog.RadioEn->setChecked(true);

	dialog.SpinRemind->setValue(RemindBy);	//    

	if (dialog.exec()) //    ,    ,  ,   
	{
		if ( FileName !=  dialog.EditFileName->text() )
		{
			QString OldFileName = FileName;			//   
			FileName = dialog.EditFileName->text();	//  
			if ( saveFile() )	//  
			{
				QFile(OldFileName).remove(); //  ,   
			}
			else
			{
				FileName = OldFileName; //        ,     
				QMessageBox::information( this, tr("Information"), tr("The new file saving failed") );
			}
		}
		
		if ( dialog.RadioDate->isChecked() )	SortBy = 1; //  
		else									SortBy = 0;
		sortAndSet(ToNowhere); //   ( )

		//  
		if ( LangRu != dialog.RadioRu->isChecked() )
		{
			QByteArray encodedString =
			"The language of the interface<br />"
			"will be changed on the next<br />"
			"start of the program.<br /><br />"
			"   <br />"
			"  .<br />";

			QTextCodec *codec = QTextCodec::codecForName("Windows-1251");
			QString st = codec->toUnicode(encodedString);
			QMessageBox::information( this, tr("Information"), st );
			LangRu = dialog.RadioRu->isChecked();
		}

		RemindBy	= dialog.SpinRemind->value();			//      
		Remind		= dialog.CheckRemind->isChecked();		//   
		ViewStyle	= dialog.ComboBoxStyle->currentText();	//  
		QApplication::setStyle( QStyleFactory::create(ViewStyle) );
		QApplication::setPalette( QApplication::style()->standardPalette() );

		writeSettings(); //  
	}
}



//       
void FormMain::remindFromForm()
{
	remind(true);
}



//  OK
void FormMain::ok()
{
	writeSettings();	//  
	saveFile();			//  
	setModified(false);	//  ,     (..,  )
	close();			//   
}



//  
void FormMain::clearTable()
{
	// ,     ?
	int r = QMessageBox::question(
				this,
				tr("Clear the table"),
				tr("Do you really want to clear the table?"),
				QMessageBox::Yes | QMessageBox::No,
				QMessageBox::No);

	//  ,  
	if ( r == QMessageBox::Yes )
	{
		Table->setRowCount(0);
		setModified(true);
	}
}



//          LABEL
void FormMain::showBD(int Y, int M)
{
	// ,     
	LabelDate->setText( getMonth(M) + ", " + QString::number(Y) );

	//       
	QFont fnt = Calendar->font();
	fnt.setBold(true);
	QTextCharFormat GreenFormat;
	GreenFormat.setFont(fnt);
	GreenFormat.setForeground(QColor(30, 174, 7)); //  

	int r = Table->rowCount(); // ,    
	if ( r == 0 ) return;
	for ( int i = 0; i < r; i++ )
	{
		//     
		QString ComputerDate = Table->item(i,1)->data(0).toString();
		int p = ComputerDate.indexOf(".");
		int m = ComputerDate.left(p).toInt();
		int d = ComputerDate.right(p).toInt();
		if ( m == M ) //       ,  
		{
			QDate date = QDate(Y, M, d);
			Calendar->setDateTextFormat(date, GreenFormat);
		}
	}
}



//     
void FormMain::showDate(const QDate date)
{
	TextEditDay->clear();
	int r = Table->rowCount(); // ,    
	bool y = false; //  -      
	if ( r != 0 )
	{
		for ( int i = 0; i < r; i++ )
		{
			//     
			QString PersonName = Table->item(i,0)->data(0).toString();
			QString ComputerDate = Table->item(i,1)->data(0).toString();
			int p = ComputerDate.indexOf(".");
			int m = ComputerDate.left(p).toInt();
			int d = ComputerDate.right(p).toInt();
			if ( ( d == date.day() ) && ( m == date.month() ) )	//          
			{													//  ,      
				if ( !y ) TextEditDay->append( "<strong>" + ProgramTitle + "</strong>:" );
				TextEditDay->append( PersonName );
				y = true;
			}
		}
	}
	if ( !y ) TextEditDay->append( "<strong>" + tr("No birthdays on this date") + "</strong>" );
}



//     
void FormMain::showMonth(int M)
{
	M++;
	TextEditMonth->clear();
	int r = Table->rowCount(); // ,    
	bool y = false; //  -      
	if ( r != 0 )
	{
		for ( int i = 0; i < r; i++ )
		{
			//     
			QString PersonName = Table->item(i,0)->data(0).toString();
			QString ComputerDate = Table->item(i,1)->data(0).toString();
			int p = ComputerDate.indexOf(".");
			int m = ComputerDate.left(p).toInt();
			int d = ComputerDate.right(p).toInt();
			if ( m == M )	//       
			{				// ,      
				if ( !y ) TextEditMonth->append( "<strong>" + tr("List of birthdays in %1").arg( getMonth(M) ) + "</strong>:");
				TextEditMonth->append( QString::number(d) + "\t" + PersonName);
				y = true;
			}
		}
	}
	if ( !y ) TextEditMonth->append( "<strong>" + tr("No birthdays in %1").arg( getMonth(M) ) + "</strong>" );
}



//    
void FormMain::tabChanged(int index)
{
	switch ( index )
	{
		case 0: //  1 ,   -   
			break;

		case 1: //  2     -      
		{
			int cr = ListMonth->currentRow();
			if ( cr < 0 )
			{
				QDate date = QDate::currentDate();
				cr = date.month() - 1;
			}
			QVector<int> count(13,0); //    13 ,  
			int r = Table->rowCount();
			if ( r != 0 )
			{
				for ( int i = 0; i < r; i++ )
				{
					//     
					QString PersonName = Table->item(i,0)->data(0).toString();
					QString ComputerDate = Table->item(i,1)->data(0).toString();
					int p = ComputerDate.indexOf(".");
					int m = ComputerDate.left(p).toInt();
					//    
					count[m] += 1;
				}
			}
			ListMonth->clear();
			for ( int i = 1; i <= 12; i++ ) ListMonth->addItem( getMonth(i) + "\t" + QString::number(count[i]) );
			ListMonth->setCurrentRow( cr );
			showMonth( cr );
			break;
		}

		case 2: //  3    -  ,        
		{
			QDate dt = Calendar->selectedDate(); //   
			QSizePolicy sp = Calendar->sizePolicy();
			//       ,   Calendar
			Calendar->~QCalendarWidget(); //  
			Calendar = new QCalendarWidget(this); //  
			Calendar->setSizePolicy(sp); //   ,     
			WidgetForCalendar->layout()->addWidget(Calendar); //   . 
			//    
			connect( Calendar, SIGNAL(clicked(const QDate)), 		this, SLOT(showDate(const QDate)) );
			connect( Calendar, SIGNAL(currentPageChanged(int, int)),this, SLOT(showBD(int, int)) );
			connect( ButtonPrevious,SIGNAL(clicked()), Calendar, SLOT(showPreviousMonth()) );
			connect( ButtonNext,	SIGNAL(clicked()), Calendar, SLOT(showNextMonth()) );
			//     
			Calendar->setFirstDayOfWeek(Qt::Monday); //     
			Calendar->setHorizontalHeaderFormat(QCalendarWidget::SingleLetterDayNames); //    
			Calendar->setVerticalHeaderFormat(QCalendarWidget::NoVerticalHeader); //   
			Calendar->setGridVisible(true); //   
			Calendar->setNavigationBarVisible(false); //   
			Calendar->setSelectedDate(dt); //   
			//            
			showBD( dt.year(), dt.month() );
			showDate( dt );
			break;
		}

		default: break; //     (   ...) -   
	}
}



//     
void FormMain::doubleClicked(int r, int c)
{
	infoPerson(); //     ,    
}



// ,    
void FormMain::setModified(bool mod)
{
	modified = mod;
}



// ,    
bool FormMain::getModified()
{
	return modified;
}



//    
void FormMain::createMenu()
{
	QAction *PAddAction = new QAction( tr("&Add"), this );				//   
	PAddAction->setIcon( QIcon(":/images/add.png") );					//    
	PAddAction->setShortcut( tr("Ctrl+N") );							//     
	connect( PAddAction, SIGNAL(triggered()), this, SLOT(addPerson()) );//    
	//       

	QAction *PDeleteAction = new QAction( tr("&Delete"), this );
	PDeleteAction->setIcon( QIcon(":/images/delete.png") );
	PDeleteAction->setShortcut( tr("Del") );
	connect( PDeleteAction, SIGNAL(triggered()), this, SLOT(deletePerson()) );

	QAction *PChangeAction = new QAction( tr("&Change"), this );
	PChangeAction->setIcon( QIcon(":/images/change.png") );
	PChangeAction->setShortcut( tr("Alt+C") );
	connect( PChangeAction, SIGNAL(triggered()), this, SLOT(changePerson()) );

	QAction *PInfoAction = new QAction( tr("&Info"), this );
	PInfoAction->setIcon( QIcon(":/images/info.png") );
	PInfoAction->setShortcut( tr("Ctrl+I") );
	connect( PInfoAction, SIGNAL(triggered()), this, SLOT(infoPerson()) );

	QAction *ClearAction = new QAction( tr("&Clear the table"), this );
	ClearAction->setIcon( QIcon(":/images/clear.png") );
	ClearAction->setShortcut( tr("Shift+Del") );
	connect( ClearAction, SIGNAL(triggered()), this, SLOT(clearTable()) );

	//      
	Table->addAction( PAddAction );
	Table->addAction( PDeleteAction );
	Table->addAction( PChangeAction );
	Table->addAction( PInfoAction );
	Table->addAction( ClearAction );
	// ,     
	Table->setContextMenuPolicy( Qt::ActionsContextMenu );
}



//        ( /)
void FormMain::setPerson(int row, QString PName, QString CDate)
{
	QTableWidgetItem *item; //   -

	//    
	item = new QTableWidgetItem(PName,0);
	Table->setItem(row,0,item);
	//       
	item = new QTableWidgetItem(CDate,0);
	Table->setItem(row,1,item);
}



//  
void FormMain::readSettings()
{
	QSettings settings; //   (   )

	//   
	QRect rect = settings.value("Geometry", QRect(345, 175, 510, 450)).toRect();
	move( rect.topLeft() ); //  
	resize( rect.size() );  //    

	LangRu		= settings.value("LangRu",		false).toBool();	// 
	RemindBy	= settings.value("RemindBy",	3).toInt();		//      
	FileName	= settings.value("FileName",	HomeDir + "birthdays.bob").toString(); //  

	ViewStyle = settings.value("Style", "WindowsVista").toString(); //  
	QStringList styles = QStyleFactory::keys();
	if ( styles.indexOf(ViewStyle) < 0 ) ViewStyle = "WindowsVista";
	QApplication::setStyle( QStyleFactory::create(ViewStyle) );
	QApplication::setPalette( QApplication::style()->standardPalette() );

	SortBy = settings.value("SortBy", 1).toInt(); //  
	if ( (SortBy != 0) and (SortBy != 1) ) SortBy = 1;

	NameWidth = settings.value("NameWidth", 220).toInt(); //    
	Table->setColumnWidth( 0, NameWidth );

	#ifdef Q_OS_WIN
	//    
	QSettings* AutorunSetting = new QSettings( "HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\Run", QSettings::NativeFormat );
	QString path1 = QDir::toNativeSeparators(QCoreApplication::applicationFilePath()) + " -r";
	QString path2 = AutorunSetting->value("birthdays", "").toString();
	path1 == path2 ? Remind = true : Remind = false; //         ,     
	#endif
}



//  
void FormMain::writeSettings()
{
	QSettings settings; //     (  )

	settings.setValue("Geometry",	geometry());	//  
	settings.setValue("SortBy",		SortBy);		//  
	settings.setValue("RemindBy",	RemindBy);		//      
	settings.setValue("LangRu",		LangRu);		//  
	settings.setValue("NameWidth",	Table->columnWidth(0)); //   
	settings.setValue("FileName",	FileName);		//  
	settings.setValue("Style",		ViewStyle);		//  

	#ifdef Q_OS_WIN //   Windows
	//     
	QSettings* AutorunSetting = new QSettings( "HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\Run", QSettings::NativeFormat );
	QString path = QDir::toNativeSeparators(QCoreApplication::applicationFilePath()) + " -r";
	if ( Remind	)
	{
		AutorunSetting->setValue( "birthdays", path ); //   ,    
	}
	else
	{
		//    ,    -     
		//  ,     ,     
		if ( path == AutorunSetting->value("birthdays", "").toString() ) AutorunSetting->remove( "birthdays" );
	}
	#endif
}



//        
void FormMain::sortAndSet(WhereSetDate WhatRow)
{
	Table->sortItems(SortBy);			//  
	Table->setColumnHidden( 1, true );	//     
	//   ,     

	switch ( WhatRow )
	{
		case ( ToAll ): //       
		{
			for ( int i = 0; i < Table->rowCount(); i++ )
			{
				QString dt = Table->item(i,1)->data(0).toString();
				int p = dt.indexOf(".");	//   .
				int m = dt.left(p).toInt();	//  ,    - 
				QString sd = dt.right(p);	//  ,    - 
				QString sm = getMonth(m);	//      
				//      2 
				QTableWidgetItem *item = new QTableWidgetItem( sm + ", " + sd ,0);
				Table->setItem( i, 2, item );
				Table->setRowHeight( i, 25 ); //    ,   
			}
			break;
		}
	
		case ( ToCurrent ): //       (   )
		{
			int i = Table->currentRow();
			QString dt = Table->item(i,1)->data(0).toString();
			int p = dt.indexOf(".");	//   .
			int m = dt.left(p).toInt();	//  ,    - 
			QString sd = dt.right(p);	//  ,    - 
			QString sm = getMonth(m);	//      
			//      2 
			QTableWidgetItem *item = new QTableWidgetItem( sm + ", " + sd ,0);
			Table->setItem( i, 2, item );
			Table->setRowHeight( i, 25 ); //    ,   
			break;
		}

		default: break; //     
	}
}



//  
bool FormMain::loadFile()
{
	QFile file(FileName);
	if (!file.open(QIODevice::ReadOnly)) return false;

	QDataStream in(&file);
	in.setVersion(QDataStream::Qt_4_4);

	qint32 magic;
	in >> magic;
	if ( magic != MagicNumber ) return false;

	QString PName, CDate;

	int last = Table->rowCount(); //     

	QApplication::setOverrideCursor(Qt::WaitCursor);
	while (!in.atEnd())
	{
		in >> PName >> CDate;			//    
		Table->setRowCount( last + 1 );	//   
		setPerson(last, PName, CDate);	//      
		last++;
	}
	QApplication::restoreOverrideCursor();
	return true;
}



//  
bool FormMain::saveFile()
{
	QFile file(FileName);
	if (!file.open(QIODevice::WriteOnly)) return false;

	QDataStream out(&file);
	out.setVersion(QDataStream::Qt_4_4);

	out << quint32(MagicNumber);

	QString str = "";

	QApplication::setOverrideCursor(Qt::WaitCursor);
	for (int row = 0; row < Table->rowCount(); ++row)
	{
		str = Table->item(row, 0)->data(0).toString(); //    
		out << str;
		str = Table->item(row, 1)->data(0).toString(); //    . 
		out << str;
	}
	QApplication::restoreOverrideCursor();
	return true;
}



//      
bool FormMain::duplicateName(QString SearchName, int ChangeItem)
{
	QString dst;
	bool f = false;

	for ( int i = 0; i < Table->rowCount(); i++ )
	{
		if ( i == ChangeItem ) continue; //   ,    ,  
		dst = Table->item(i,0)->data(0).toString(); //   
		if ( dst == SearchName )
		{
			QMessageBox::information
			(
				this,
				tr("Information"),
				tr("%1 is already in the table").arg("<strong>" + SearchName + "</strong>")
			);
			f = true;
			break;
		}
	}
	return f;
}



//    ,        
void FormMain::infoRow()
{
	QString message = tr("You must select a row");
	if ( Table->rowCount() < 1 ) //      
	{
		message = tr("There's no entry in the table");
	}
	QMessageBox::information( this, tr("Information"), message );
}



//       (  )
QString FormMain::getMonth(int m)
{
	QVector<QString> Months;
	Months << ""
		   << tr("January")
		   << tr("February")
		   << tr("March")
		   << tr("April")
		   << tr("May")
		   << tr("June")
		   << tr("July")
		   << tr("August")
		   << tr("September")
		   << tr("October")
		   << tr("November")
		   << tr("December");

	return Months.at(m);
}



//   
QString FormMain::zodiac(int m, int d)
{
	QStringList zs; //    
	zs << ""
	<< tr("Capricorn")		// 
	<< tr("Aquarius")		// 
	<< tr("Pisces")			// 
	<< tr("Aries")			// 
	<< tr("Taurus")			// 
	<< tr("Gemini")			// 
	<< tr("Cancer")			// 
	<< tr("Leo")			// 
	<< tr("Virgo")			// 	
	<< tr("Libra")			// 
	<< tr("Scorpio")		// 
	<< tr("Sagittarius");	// 

	switch (m)
	{
		case 2:				if ( !( ( d >= 1 ) and ( d <= 18 ) ) ) m++; break;
		case 4: case  1:	if ( !( ( d >= 1 ) and ( d <= 19 ) ) ) m++; break;
		case 3: case  5:	if ( !( ( d >= 1 ) and ( d <= 20 ) ) ) m++; break;
		case 6: case 12:	if ( !( ( d >= 1 ) and ( d <= 21 ) ) ) m++; break;
		default:			if ( !( ( d >= 1 ) and ( d <= 22 ) ) ) m++; break;
	}

	if ( m > 12 ) m = 1;
	return zs.at(m);
}



//    
int FormMain::daysToBirthday(int m, int d)
{
	int dt; // ,        

	QDate date = QDate::currentDate(); //   
	QDate know;
	int y = date.year();

	//  29 
	if ( (m == 2) && (d == 29) )
	{
		while ( !QDate::isLeapYear(y) ) y++;
		know = QDate( y, m, d ); //      
		if ( know < date ) //       
		{
			y++;
			while ( !QDate::isLeapYear(y) ) y++;
			know = QDate( y, m, d );
		}
		dt = date.daysTo(know); // ,     
		return dt;
	}

	//    
	know = QDate( y, m, d );	//      
	if ( know < date ) know = QDate( date.year() + 1, m, d ); //       ,     
	dt = date.daysTo(know); // ,     

	return dt;
}
